#version 150
#extension GL_EXT_gpu_shader4 : enable
#define MAX_ACTIVE_LIGHTS 4

uniform vec4 lightPositions[MAX_ACTIVE_LIGHTS];
uniform vec4 lightColors[MAX_ACTIVE_LIGHTS];
//uniform vec4 lightDirections[MAX_ACTIVE_LIGHTS];
//uniform float numActiveLights;
//uniform vec3 cameraPosition;

uniform sampler2DArray colorArray;
uniform sampler2DArray detailArray;

in vec3 vViewVec;
in vec3 vEyeVec;
in vec4 vColor;
in vec4 vTexCoord;
in vec4 vLightVector[MAX_ACTIVE_LIGHTS];
//in vec3 vLightDistance[MAX_ACTIVE_LIGHTS];

out vec4 fragmentColor;

void main(){
	// normalize vertex vector
	vec3 eyeVec = normalize(vEyeVec);
	
	// read height data form normal map
	float height = texture2DArray(detailArray, vTexCoord.xyz).a * 2.0*vTexCoord.w - vTexCoord.w;
	// compute new texture position
	vec3 texCoord = vec3(vTexCoord.xy + eyeVec.xy * height, vTexCoord.z);
	// read new normal data from normal map
	vec4 detail = texture2DArray(detailArray, texCoord.xyz);
	// compute normal
	detail.xyz = normalize(detail.xyz - 0.5);
	
	// read color form color texture
	vec4 texColor = texture2DArray(colorArray, texCoord.xyz);
	
	// compute sun values
	vec4 color = vec4(0.0, 0.0, 0.0, 0.0);
			
	// compute light values
	for(int i=0; i<MAX_ACTIVE_LIGHTS; i++){
		//if(vLightVector[i].w * dot(normalize(vLightDistance[i].xyz), lightDirections[i].xyz) > lightDirections[i].w){
			// normalize light vector for phong effect
			vec3 normLightVec = normalize(vLightVector[i].xyz);
			// compute the amount of diffuse light
			// that is reflected by surface
			float diffuse = max(dot(normLightVec, detail.xyz), 0.0);
			// compute the specular factor
			float specular = pow(max(dot(reflect(-eyeVec, detail.xyz), normLightVec), 0.0), vColor.w);
			// compute the combined color of the light and the surface
			// lightColors[i].w ... inverse radius
			vec3 c = vColor.xyz * lightColors[i].xyz * max(0.0, 1.0-length(vLightVector[i].xyz)*lightColors[i].w);
			// combine all values to a single color value and
			// add it to the current color
			color += vec4(c, 1.0) * (diffuse * detail.w + specular);
		//}
	}
	
	fragmentColor = vec4(texColor.xyz * color.xyz, 0.0027*length(vViewVec));
}

