#version 150
#define MAX_ACTIVE_LIGHTS 4

uniform mat4 viewMat;
uniform mat4 modelViewMat;
uniform mat3 normalMat;
uniform mat4 projMat;
// texIdx.x ... texture index
// texIdx.y ... texture bump height
uniform vec2 texIdx;
uniform vec4 lightPositions[MAX_ACTIVE_LIGHTS];
//uniform float numActiveLights;
//uniform vec3 cameraPosition;

// vertex input
in vec4 position;
in vec3 normal;
in vec3 tangent;
in vec3 binormal;
in vec4 color;
in vec4 uv;
in vec4 bones;
in vec4 weights;

// vertex output
out vec3 vViewVec;
out	vec3 vEyeVec;
out vec4 vColor;
out vec4 vTexCoord;
out vec4 vLightVector[MAX_ACTIVE_LIGHTS];
out vec3 vLightDistance[MAX_ACTIVE_LIGHTS];

void main(){
	//mat3 normalMat = mat3(modelViewMat);
	
	// compute normal space rotation matrix
	mat3 tbnMatrix = normalMat * mat3(tangent, binormal, normal);
	
	// transform vertex into model view space
	vec4 vertex = modelViewMat * position;
	// transform vertex into model view projection space
	gl_Position = projMat * vertex;
	// vector from the camera to the vertex
	vViewVec = -vertex.xyz;
	// transform vertex into normal space
	vEyeVec = -vertex.xyz * tbnMatrix;
	// vertex color
	vColor = color;
	// texture coordinates, texture index, and bump height
	vTexCoord = vec4(uv.xy, texIdx.xy);
	
	// for all lights
	for(int i=0; i<MAX_ACTIVE_LIGHTS; i++){
		// vector to the light source in normal coordinates
		vLightVector[i] = vec4(((viewMat * lightPositions[i]).xyz - vertex.xyz) * tbnMatrix, lightPositions[i].w);
		// distance from the light to the vertex
		//vLightDistance[i] = position.xyz - lightPositions[i].xyz;
	}
}


